This module contains definitions for the HTTP types that the user interacts with.
It provides a RequestBuilder that allows constructing requests
as well as the Response type used to receive responses using a Client.
The SimpleClient uses it’s own response types.
Structs
- Header
- An HTTP header.
- Owned
Header - An owned HTTP header. This is used in a response.
- Query
- An HTTP query.
- RawRequest
- ReqId
- The ID assigned to a request.
- Request
- Represents an HTTP request.
You can build a request either through this struct directly
or through a
RequestBuilder. - Request
Builder - Used to build a request.
See
Request. - Response
- An HTTP response. Contains a
ResponseState. - Response
Head - The
Headof a response. This is not to be confused with an HTTPHeader. - Status
- A status code and message for a response.
- Uri
- An HTTP URI.
The path may start with a
/or it may not.
Enums
- Method
- An HTTP method.
The default method is
GET. - Mode
- If the connection should use tls or not.
- Response
State - The state of a response.